﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win.UltraWinDock;

namespace WindowMaxMinDockMgr
{
    public partial class Form1 : Form
    {
        private DockStructureObserver m_DockStructureObserver;

        private int m_WindowCounter = 0;

        public Form1()
        {
            InitializeComponent();
        }

        private DockerPanel DockerPanel { get { return this.dockerPanel; } }

        private bool WindowManagerAlwaysOnTop { get { return windowManagerAlwaysOnTopToolStripMenuItem.Checked; } }
        private bool HideMinimizedWindows { get { return hideMinimizedWindowsToolStripMenuItem.Checked; } }

        private string AssignWindowName(string prefix)
        {
            return prefix + " " + (++m_WindowCounter).ToString();
        }

        private int m_NextPanelNumber = 0;
        private void AdvancePanelNumber() { ++m_NextPanelNumber; }
        private int GetCurrentPanelNumber() { return m_NextPanelNumber; }
        private string FormatPanelCaption(XPanel panel, int id)
        {
            return string.Format("Doc {0}", id);
        }

        private Control InitializeContent()
        {
            PictureBox pb = new PictureBox();
            pb.Dock = DockStyle.Fill;
            pb.Image = Image.FromFile("Images/8095.jpg");
            return pb;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            try
            {
                XPanel panel = DockerPanel.CreatePanel();
                Control content = InitializeContent();
                panel.SetContent(content);

                AdvancePanelNumber();
                int id = GetCurrentPanelNumber();
                string caption = FormatPanelCaption(panel, id);
                DockerPanel.DockPanel(DockerPanel.DockManager, panel, caption);
                NotifyDockStructureObserver(DockerPanel.DockManager);
            }
            catch (Exception ex)
            {
                string msg = "Exception: " + ex.Message;
                MessageBox.Show(msg);
            }
        }

        private void OnLoad(object sender, EventArgs e)
        {
            DockerPanel.HideMinimizedWindows = HideMinimizedWindows;
            m_DockStructureObserver = new DockStructureObserver(DockerPanel, WindowManagerAlwaysOnTop);
            m_DockStructureObserver.ShowInTaskbar = true;
        }

        private void NotifyDockStructureObserver(UltraDockManager dm)
        {
            if (!m_DockStructureObserver.Visible)
            {
                m_DockStructureObserver.Show();
                SyncDockIndicatorsOption();
            }
        }

        private void SyncDockIndicatorsOption()
        {
            if (showAllowedIndicatorsByValidationToolStripMenuItem.Checked)
            {
                OnShowAllowedDockIndicatorsByValidation(this, EventArgs.Empty);
            }
            else if (showAllowedIndicatorsByConfigurationToolStripMenuItem.Checked)
            {
                OnShowAllowedDockIndicatorsByConfiguration(this, EventArgs.Empty);
            }
            else
            {
                Debug.Assert(showAllIndicatorsToolStripMenuItem.Checked);
                OnShowAllDockIndicators(this, EventArgs.Empty);
            }
        }

        private void OnWindowsManagerOnTopChanged(object sender, EventArgs e)
        {
            m_DockStructureObserver.TopMost = WindowManagerAlwaysOnTop;
        }

        private void OnHideMinimizedWindowsChanged(object sender, EventArgs e)
        {
            DockerPanel.HideMinimizedWindows = HideMinimizedWindows;
        }

        private void OnShowAllDockIndicators(object sender, EventArgs e)
        {
            showAllIndicatorsToolStripMenuItem.Checked = true;
            showAllowedIndicatorsByValidationToolStripMenuItem.Checked = false;
            showAllowedIndicatorsByConfigurationToolStripMenuItem.Checked = false;

            DockerPanel.ShowAllDockIndicators();
        }

        private void OnShowAllowedDockIndicatorsByValidation(object sender, EventArgs e)
        {
            showAllIndicatorsToolStripMenuItem.Checked = false;
            showAllowedIndicatorsByValidationToolStripMenuItem.Checked = true;
            showAllowedIndicatorsByConfigurationToolStripMenuItem.Checked = false;

            DockerPanel.ShowAllowedDockIndicatorsByValidation();
        }

        private void OnShowAllowedDockIndicatorsByConfiguration(object sender, EventArgs e)
        {
            showAllIndicatorsToolStripMenuItem.Checked = false;
            showAllowedIndicatorsByValidationToolStripMenuItem.Checked = false;
            showAllowedIndicatorsByConfigurationToolStripMenuItem.Checked = true;

            DockerPanel.ShowAllowedDockIndicatorsByConfiguration();
        }

        private void OnFormClosing(object sender, FormClosingEventArgs e)
        {
            DockerPanel.CloseAllPanes();
        }
    }
}
