﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace WindowMaxMinDockMgr
{
    public partial class PaneProxyForm : Form
    {
        #region Member variables
        private bool m_HideWhenMinimized = true;
        #endregion Member variables

        #region Events and helper methods and classes
        public event EventHandler PaneResize;
        private void RaisePaneResizeEvent()
        {
            if (PaneResize != null)
            {
                try
                {
                    PaneResize(this, EventArgs.Empty);
                }
                catch (Exception ex)
                {
                    string msg = "Exception in RaisePaneResizeEvent: " + ex.GetType().FullName + ": " + ex.Message;
                    MessageBox.Show(msg);
                }
            }
        }

        public event EventHandler DestroyingForm;
        private void RaiseDestroyingFormEvent()
        {
            if (DestroyingForm != null)
            {
                try
                {
                    DestroyingForm(this, EventArgs.Empty);
                }
                catch (Exception ex)
                {
                    string msg = "Exception in RaiseDestroyingFormEvent: " + ex.GetType().FullName + ": " + ex.Message;
                    MessageBox.Show(msg);
                }
            }
        }
        #endregion Events and helper methods and classes

        #region Constructor
        public PaneProxyForm(object pane, bool hideWhenMinimized)
        {
            InitializeComponent();

            Pane = pane;
            Loaded = false;
            Resizing = false;
            Destroying = false;
            Visible = false;

            HideWhenMinimized = hideWhenMinimized;
        }
        #endregion Constructor

        #region Properties
        private bool Loaded { get; set; }
        public bool Resizing { get; set; }
        public object Pane { get; private set; }
        public PaneState.PaneStateValue State 
        {
            get { return PaneStateFromWindowState(WindowState); }
            set
            {
                FormWindowState newState = WindowStateFromPaneState(value);
                if (WindowState != newState) WindowState = WindowStateFromPaneState(value);
                EnforceHideOnMinimizePolicy();
            }
        }

        public bool HideWhenMinimized
        {
            get { return m_HideWhenMinimized; }
            set
            {
                m_HideWhenMinimized = value;
                EnforceHideOnMinimizePolicy();
            }
        }

        private bool Destroying { get; set; }
        #endregion Properties

        #region Helper methods
        private void EnforceHideOnMinimizePolicy()
        {
            if (Loaded)
            {
                if (State == PaneState.PaneStateValue.Minimized)
                {
                    string msg = "Setting '" + Text + "'.Visible to " + (!m_HideWhenMinimized).ToString();
                    Debug.WriteLine(msg);
                    Visible = !m_HideWhenMinimized;
                }
            }
        }

        public void SetContent(Control content)
        {
            Controls.Add(content);
        }

        public Control GetContent()
        {
            if (Controls.Count > 0) return Controls[0]; else return null;
        }

        public void ClearContent()
        {
            Controls.Clear();
        }

        public string DumpState()
        {
            string msg = "'" + Text + "':";
            msg += " WindowState=" + Enum.GetName(typeof(FormWindowState), WindowState);
            msg += " Visible=" + Visible.ToString();
            return msg;
        }

        private PaneState.PaneStateValue PaneStateFromWindowState(FormWindowState windowState)
        {
            if (windowState == FormWindowState.Minimized) return PaneState.PaneStateValue.Minimized;
            if (windowState == FormWindowState.Maximized) return PaneState.PaneStateValue.Maximized;
            return PaneState.PaneStateValue.Normal;
        }

        private FormWindowState WindowStateFromPaneState(PaneState.PaneStateValue paneState)
        {
            if (paneState == PaneState.PaneStateValue.Minimized) return FormWindowState.Minimized;
            if (paneState == PaneState.PaneStateValue.Maximized) return FormWindowState.Maximized;
            return FormWindowState.Normal;
        }

        public void Destroy()
        {
            Destroying = true;
            Close();
        }
        #endregion Helper methods

        #region Event handlers
        private void OnLoad(object sender, EventArgs e)
        {
            Loaded = true;
            EnforceHideOnMinimizePolicy();
        }

        private void OnResize(object sender, EventArgs e)
        {
            Debug.WriteLine("OnResize: " + DumpState());
            RaisePaneResizeEvent();
        }

        private void OnFormClosing(object sender, FormClosingEventArgs e)
        {
            if (!Destroying)
            {
                e.Cancel = true;
                Hide();
            }

            RaiseDestroyingFormEvent();
        }
        #endregion Event handlers
    }
}
