﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;

namespace WindowMaxMinDockMgr
{
    public class Tuple<T1, T2>
    {
        private readonly T1 _t1;
        private readonly T2 _t2;
        public Tuple(T1 t1, T2 t2) { _t1 = t1; _t2 = t2; }
        public T1 Item1 { get { return _t1; } }
        public T2 Item2 { get { return _t2; } }
    }

    public class Tuple<T1, T2, T3>
    {
        private readonly T1 _t1;
        private readonly T2 _t2;
        private readonly T3 _t3;
        public Tuple(T1 t1, T2 t2, T3 t3) { _t1 = t1; _t2 = t2; _t3 = t3; }
        public T1 Item1 { get { return _t1; } }
        public T2 Item2 { get { return _t2; } }
        public T3 Item3 { get { return _t3; } }
    }

    public class Utilities
    {
        // Zips together two lists into a list of 2-tuples. Resulting list contains
        // the fewer number of entries of the arguments.
        public static List<Tuple<T1, T2>> Zip<T1, T2>(List<T1> t1s, List<T2> t2s)
        {
            List<Tuple<T1, T2>> result = new List<Tuple<T1, T2>>();
            int n = Math.Min(t1s.Count, t2s.Count);
            for (int i = 0; i < n; ++i)
                result.Add(new Tuple<T1, T2>(t1s[i], t2s[i]));
            return result;
        }

        public static List<Tuple<Point, Size>> SplitHorizontally(int n, Point pt, Size sz)
        {
            List<Tuple<Point, Size>> list = new List<Tuple<Point, Size>>();
            Size tileSize = new Size(sz.Width / n, sz.Height);
            for (int i = 0; i < n; ++i)
                list.Add(new Tuple<Point, Size>(new Point(pt.X + i * tileSize.Width, pt.Y), tileSize));
            return list;
        }

        public static List<Tuple<Point, Size>> SplitVertically(int n, Point pt, Size sz)
        {
            List<Tuple<Point, Size>> list = new List<Tuple<Point, Size>>();
            Size tileSize = new Size(sz.Width, sz.Height / n);
            for (int i = 0; i < n; ++i)
                list.Add(new Tuple<Point, Size>(new Point(pt.X, pt.Y + i * tileSize.Height), tileSize));
            return list;
        }

        public static string GetCallingMethodName(int frameCount)
        {
            StackTrace stackTrace = new StackTrace();

            string callerClass = stackTrace.GetFrame(frameCount).GetMethod().ReflectedType.FullName;
            string callerName = stackTrace.GetFrame(frameCount).GetMethod().Name;
            return callerClass + "." + callerName;
        }

        public static void WithIndent(Action action)
        {
            try { Debug.Indent(); action(); }
            finally { Debug.Unindent(); }
        }

        public static void NestedTrace(Action action)
        {
            Debug.WriteLine(GetCallingMethodName(2));
            WithIndent(() => { action(); });
        }

        public static void NestedTrace(Action action, Action finallyAction)
        {
            Debug.WriteLine(GetCallingMethodName(2));
            WithIndent(() => { try { action(); } finally { finallyAction(); } });
        }
    }
}
